#include "utils.h"
#include "Max7219.h"

void ADCInit(void)
{
    uint16_t result;
 
  ADCSRA = (1<<ADPS1) | (1<<ADPS2);     // Frequenzvorteiler
  ADCSRA |= (1<<ADEN);                  // ADC aktivieren
 
  /* nach Aktivieren des ADC wird ein "Dummy-Readout" empfohlen, man liest
     also einen Wert und verwirft diesen, um den ADC "warmlaufen zu lassen" */
 
  ADCSRA |= (1<<ADSC);                  // eine ADC-Wandlung 
  while (ADCSRA & (1<<ADSC) );          // auf Abschluss der Konvertierung warten
  /* ADCW muss einmal gelesen werden, sonst wird Ergebnis der nchsten
     Wandlung nicht bernommen. */
  result = ADCW;
}
 
/* ADC Einzelmessung */
uint16_t ADC_Read( uint8_t channel )
{
  // Kanal waehlen, ohne andere Bits zu beeinfluen
  ADMUX = (ADMUX & ~(0x1F)) | (channel & 0x1F);
  ADCSRA |= (1<<ADSC);            // eine Wandlung "single conversion"
  while (ADCSRA & (1<<ADSC) )     // auf Abschluss der Konvertierung warten
    ;
  return ADCW;                    // ADC auslesen und zurckgeben

}

const uint8_t keys[] = {6,15,32,42,80,105,122,156,190};

uint8_t count = 0;
uint8_t key = 0;

uint8_t GetKey(void)
{
    uint8_t x = ADC_Read(0) / 4;
    if(x > keys[0])
    {
        uint8_t i = 1;
        while(i < 9 && x > keys[i]) i++;
            if(key != i)
                count = 0;
        key = i;
        count++;
        if(count >= 100)
        {
            count = 0;
            return i;   
        }
        return 255;
    }
    return 0;
}

void KeyTest(void)
{
    SetAll(128);
    while(1)
    {
        uint8_t key = GetKey();
        if(key != 255)
            SetDigit(0,chars[key]);
    }
}

void WaitForKeyUp(void)
{
    uint8_t count = 0;
    while(count < 100)
    {
        uint8_t key = GetKey();
        if(key == 0) 
            count++;
        else if(key != 255)
            count = 0;
    }
}
